import { IAccount } from "../data/DataModel";
import { aesDecrypt, aesEncrypt } from "../util/crypto.util";
import sha256 from "crypto-js/sha256";

const LOCAL_ACCOUNT_DATAS = "local_account_datas";
const LOCAL_WALLET_TOKEN = "local_wallet_token";
const LOCAL_WALLET_MNEMONIC = "local_wallet_mnemonic";
const LOCAL_WALLET_PASSWORD = "local_wallet_password";
const LOCAL_PASSWORD_SALT = "_jcwallet";

export function loadToken() {
  return localStorage.getItem(LOCAL_WALLET_TOKEN);
}

export function saveToken(token: string) {
  localStorage.setItem(LOCAL_WALLET_TOKEN, token);
}

export function loadData() {
  const dataStr = localStorage.getItem(LOCAL_ACCOUNT_DATAS);
  let result: IAccount[] = [];
  if (dataStr) {
    try {
      result = JSON.parse(dataStr);
    } catch (err) {
      console.log("load local data error");
    }
  }
  return result;
}

export function saveData(datas: IAccount[]) {
  const dataStr = JSON.stringify(datas);
  localStorage.setItem(LOCAL_ACCOUNT_DATAS, dataStr);
}

export function loadMnemonic(password: string) {
  let dataStr: string = localStorage.getItem(LOCAL_WALLET_MNEMONIC);
  if (dataStr) {
    dataStr = aesDecrypt(dataStr, password);
  }
  return dataStr;
}

export function saveMnemonic(mnemonic: string, password: string) {
  const dataStr = aesEncrypt(mnemonic, password);
  localStorage.setItem(LOCAL_WALLET_MNEMONIC, dataStr);
}

export function checkPassword(password: string) {
  const localStr: string = localStorage.getItem(LOCAL_WALLET_PASSWORD);
  const dataStr = sha256(password + LOCAL_PASSWORD_SALT).toString();
  return !localStr || localStr === dataStr;
}

export function savePassword(password: string) {
  const dataStr = sha256(password + LOCAL_PASSWORD_SALT).toString();
  localStorage.setItem(LOCAL_WALLET_PASSWORD, dataStr);
}
