import { AES, enc, mode, pad } from "crypto-js";
import sha256 from "crypto-js/sha256";
import sha1 from "crypto-js/sha1";
import md5 from "crypto-js/md5";

export function sha1Hash(str: string) {
  return sha1(str).toString();
}

export function md5Hash(str: string) {
  return md5(str).toString();
}

function generateIV(password: string) {
  const key = sha256(password).toString();
  const keyHex = enc.Base64.parse(key);
  const ivHex = keyHex.clone();
  ivHex.sigBytes = 16;
  ivHex.words.splice(4);
  return { keyHex, ivHex };
}

export function aesEncrypt(text: string, password: string) {
  const { keyHex, ivHex } = generateIV(password);
  const messageHex = enc.Utf8.parse(text);
  const encrypted = AES.encrypt(messageHex, keyHex, {
    iv: ivHex,
    mode: mode.CBC,
    padding: pad.Pkcs7,
  });
  return encrypted.toString();
}

export function aesDecrypt(encryptedText: string, password: string) {
  const { keyHex, ivHex } = generateIV(password);
  const decrypt = AES.decrypt(encryptedText, keyHex, {
    iv: ivHex,
    mode: mode.CBC,
    padding: pad.Pkcs7,
  });
  return enc.Utf8.stringify(decrypt);
}
