/**
 *
 * @param {Function} fn
 * @param {number} maxRetries
 * @param {any[]} errorWhiteList
 * @param {number} retries
 * @return {Promise<T>}
 */
export function retry<T = any>(
  fn: Function,
  maxRetries: number = 3,
  errorWhiteList: any[] = [],
  retries: number = 0
) {
  return new Promise<T>((resolve, reject) => {
    fn()
      .then(resolve)
      .catch((e: any) => {
        if (
          (errorWhiteList.length == 0 ||
            errorWhiteList.indexOf(e.constructor) !== -1) &&
          retries++ < maxRetries
        ) {
          setTimeout(() => {
            retry<T>(fn, maxRetries, errorWhiteList, retries)
              .then(resolve)
              .catch((e2) => reject(e2));
          }, Math.floor(Math.random() * Math.pow(2, retries) * 400));
        } else {
          reject(e);
        }
      });
  });
}
